# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.WalletSummaryOptions import WalletSummaryOptions


class WalletSummary:
    paid: int
    free: int
    total: int

    def __init__(
        self,
        paid: int,
        free: int,
        total: int,
        options: Optional[WalletSummaryOptions] = WalletSummaryOptions(),
    ):
        self.paid = paid
        self.free = free
        self.total = total

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.paid is not None:
            properties["paid"] = self.paid
        if self.free is not None:
            properties["free"] = self.free
        if self.total is not None:
            properties["total"] = self.total

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.func import GetAttr, Join
from .GlobalMessageRef import GlobalMessageRef
from ..stamp_sheet.SendMessageByUserId import SendMessageByUserId
from ...core.model import AcquireAction
from ..model.TimeSpan import TimeSpan
from ..stamp_sheet.OpenMessageByUserId import OpenMessageByUserId
from ..stamp_sheet.DeleteMessageByUserId import DeleteMessageByUserId


class NamespaceRef:
    namespace_name: str

    def __init__(
        self,
        namespace_name: str,
    ):
        self.namespace_name = namespace_name

    def global_message(
        self,
        global_message_name: str,
    ) -> GlobalMessageRef:
        return GlobalMessageRef(
            self.namespace_name,
            global_message_name,
        )

    def send_message(
        self,
        metadata: str,
        read_acquire_actions: Optional[List[AcquireAction]] = None,
        expires_at: Optional[int] = None,
        expires_time_span: Optional[TimeSpan] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> SendMessageByUserId:
        return SendMessageByUserId(
            self.namespace_name,
            metadata,
            read_acquire_actions,
            expires_at,
            expires_time_span,
            user_id,
        )

    def open_message(
        self,
        message_name: Optional[str] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> OpenMessageByUserId:
        return OpenMessageByUserId(
            self.namespace_name,
            message_name,
            user_id,
        )

    def delete_message(
        self,
        message_name: Optional[str] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> DeleteMessageByUserId:
        return DeleteMessageByUserId(
            self.namespace_name,
            message_name,
            user_id,
        )

    def grn(
        self,
    ) -> str:
        return Join(
            ":",
            [
                "grn",
                "gs2",
                GetAttr.region(
                ).str(
                ),
                GetAttr.owner_id(
                ).str(
                ),
                "inbox",
                self.namespace_name,
            ],
        ).str(
        )



from .StateMachine import StateMachineDefinition
from .StateMachine import StateMachine
from .StateMachine import Task
from .StateMachine import Event
from .StateMachine import ErrorEvent
from .StateMachine import Variable



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .enum.NamespaceCurrencyUsagePriority import NamespaceCurrencyUsagePriority
from .StoreContentModel import StoreContentModel
from .options.StoreContentModelOptions import StoreContentModelOptions
from .Receipt import Receipt
from .options.ReceiptOptions import ReceiptOptions
from .enum.ReceiptStore import ReceiptStore
from .PlatformSetting import PlatformSetting
from .options.PlatformSettingOptions import PlatformSettingOptions
from .AppleAppStoreSetting import AppleAppStoreSetting
from .options.AppleAppStoreSettingOptions import AppleAppStoreSettingOptions
from .GooglePlaySetting import GooglePlaySetting
from .options.GooglePlaySettingOptions import GooglePlaySettingOptions
from .FakeSetting import FakeSetting
from .options.FakeSettingOptions import FakeSettingOptions
from .enum.FakeSettingAcceptFakeReceipt import FakeSettingAcceptFakeReceipt
from .WalletSummary import WalletSummary
from .options.WalletSummaryOptions import WalletSummaryOptions
from .DepositTransaction import DepositTransaction
from .options.DepositTransactionOptions import DepositTransactionOptions
from .VerifyReceiptEvent import VerifyReceiptEvent
from .options.VerifyReceiptEventOptions import VerifyReceiptEventOptions
from .enum.VerifyReceiptEventPlatform import VerifyReceiptEventPlatform
from .DepositEvent import DepositEvent
from .options.DepositEventOptions import DepositEventOptions
from .WithdrawEvent import WithdrawEvent
from .options.WithdrawEventOptions import WithdrawEventOptions
from .AppleAppStoreVerifyReceiptEvent import AppleAppStoreVerifyReceiptEvent
from .options.AppleAppStoreVerifyReceiptEventOptions import AppleAppStoreVerifyReceiptEventOptions
from .enum.AppleAppStoreVerifyReceiptEventEnvironment import AppleAppStoreVerifyReceiptEventEnvironment
from .GooglePlayVerifyReceiptEvent import GooglePlayVerifyReceiptEvent
from .options.GooglePlayVerifyReceiptEventOptions import GooglePlayVerifyReceiptEventOptions
from .AppleAppStoreContent import AppleAppStoreContent
from .options.AppleAppStoreContentOptions import AppleAppStoreContentOptions
from .GooglePlayContent import GooglePlayContent
from .options.GooglePlayContentOptions import GooglePlayContentOptions
from .DailyTransactionHistory import DailyTransactionHistory
from .options.DailyTransactionHistoryOptions import DailyTransactionHistoryOptions
from .UnusedBalance import UnusedBalance
from .options.UnusedBalanceOptions import UnusedBalanceOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .Position import Position
from .Vector import Vector
from .options.MyPositionOptions import MyPositionOptions


class MyPosition:
    position: Position
    vector: Vector
    r: float

    def __init__(
        self,
        position: Position,
        vector: Vector,
        r: float,
        options: Optional[MyPositionOptions] = MyPositionOptions(),
    ):
        self.position = position
        self.vector = vector
        self.r = r

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.position is not None:
            properties["position"] = self.position.properties(
            )
        if self.vector is not None:
            properties["vector"] = self.vector.properties(
            )
        if self.r is not None:
            properties["r"] = self.r

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .FixedTiming import FixedTiming
from .Scope import Scope
from .options.GlobalRankingSettingOptions import GlobalRankingSettingOptions


class GlobalRankingSetting:
    unique_by_user_id: bool
    calculate_interval_minutes: int
    calculate_fixed_timing: Optional[FixedTiming] = None
    additional_scopes: Optional[List[Scope]] = None
    ignore_user_ids: Optional[List[str]] = None
    generation: Optional[str] = None

    def __init__(
        self,
        unique_by_user_id: bool,
        calculate_interval_minutes: int,
        options: Optional[GlobalRankingSettingOptions] = GlobalRankingSettingOptions(),
    ):
        self.unique_by_user_id = unique_by_user_id
        self.calculate_interval_minutes = calculate_interval_minutes
        self.calculate_fixed_timing = options.calculate_fixed_timing if options.calculate_fixed_timing else None
        self.additional_scopes = options.additional_scopes if options.additional_scopes else None
        self.ignore_user_ids = options.ignore_user_ids if options.ignore_user_ids else None
        self.generation = options.generation if options.generation else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.unique_by_user_id is not None:
            properties["uniqueByUserId"] = self.unique_by_user_id
        if self.calculate_interval_minutes is not None:
            properties["calculateIntervalMinutes"] = self.calculate_interval_minutes
        if self.calculate_fixed_timing is not None:
            properties["calculateFixedTiming"] = self.calculate_fixed_timing.properties(
            )
        if self.additional_scopes is not None:
            properties["additionalScopes"] = [
                v.properties(
                )
                for v in self.additional_scopes
            ]
        if self.ignore_user_ids is not None:
            properties["ignoreUserIds"] = self.ignore_user_ids
        if self.generation is not None:
            properties["generation"] = self.generation

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.func import GetAttr, Join
from .CategoryModelRef import CategoryModelRef
from ..stamp_sheet.IncreaseMaximumIdleMinutesByUserId import IncreaseMaximumIdleMinutesByUserId
from ..stamp_sheet.SetMaximumIdleMinutesByUserId import SetMaximumIdleMinutesByUserId
from ..stamp_sheet.ReceiveByUserId import ReceiveByUserId
from ...core.model import Config
from ..stamp_sheet.DecreaseMaximumIdleMinutesByUserId import DecreaseMaximumIdleMinutesByUserId


class NamespaceRef:
    namespace_name: str

    def __init__(
        self,
        namespace_name: str,
    ):
        self.namespace_name = namespace_name

    def category_model(
        self,
        category_name: str,
    ) -> CategoryModelRef:
        return CategoryModelRef(
            self.namespace_name,
            category_name,
        )

    def increase_maximum_idle_minutes(
        self,
        category_name: str,
        increase_minutes: Optional[int] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> IncreaseMaximumIdleMinutesByUserId:
        return IncreaseMaximumIdleMinutesByUserId(
            self.namespace_name,
            category_name,
            increase_minutes,
            user_id,
        )

    def set_maximum_idle_minutes(
        self,
        category_name: str,
        maximum_idle_minutes: Optional[int] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> SetMaximumIdleMinutesByUserId:
        return SetMaximumIdleMinutesByUserId(
            self.namespace_name,
            category_name,
            maximum_idle_minutes,
            user_id,
        )

    def receive(
        self,
        category_name: str,
        config: Optional[List[Config]] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> ReceiveByUserId:
        return ReceiveByUserId(
            self.namespace_name,
            category_name,
            config,
            user_id,
        )

    def decrease_maximum_idle_minutes(
        self,
        category_name: str,
        decrease_minutes: Optional[int] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> DecreaseMaximumIdleMinutesByUserId:
        return DecreaseMaximumIdleMinutesByUserId(
            self.namespace_name,
            category_name,
            decrease_minutes,
            user_id,
        )

    def grn(
        self,
    ) -> str:
        return Join(
            ":",
            [
                "grn",
                "gs2",
                GetAttr.region(
                ).str(
                ),
                GetAttr.owner_id(
                ).str(
                ),
                "idle",
                self.namespace_name,
            ],
        ).str(
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.func import GetAttr, Join
from .GradeModelRef import GradeModelRef
from ..stamp_sheet.AddGradeByUserId import AddGradeByUserId
from ..stamp_sheet.ApplyRankCapByUserId import ApplyRankCapByUserId
from ..stamp_sheet.MultiplyAcquireActionsByUserId import MultiplyAcquireActionsByUserId
from ...core.model import AcquireAction
from ..stamp_sheet.SubGradeByUserId import SubGradeByUserId
from ..stamp_sheet.VerifyGradeByUserId import VerifyGradeByUserId
from ..stamp_sheet.VerifyGradeUpMaterialByUserId import VerifyGradeUpMaterialByUserId


class NamespaceRef:
    namespace_name: str

    def __init__(
        self,
        namespace_name: str,
    ):
        self.namespace_name = namespace_name

    def grade_model(
        self,
        grade_name: str,
    ) -> GradeModelRef:
        return GradeModelRef(
            self.namespace_name,
            grade_name,
        )

    def add_grade(
        self,
        grade_name: str,
        property_id: str,
        grade_value: Optional[int] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> AddGradeByUserId:
        return AddGradeByUserId(
            self.namespace_name,
            grade_name,
            property_id,
            grade_value,
            user_id,
        )

    def apply_rank_cap(
        self,
        grade_name: str,
        property_id: str,
        user_id: Optional[str] = "#{userId}",
    ) -> ApplyRankCapByUserId:
        return ApplyRankCapByUserId(
            self.namespace_name,
            grade_name,
            property_id,
            user_id,
        )

    def multiply_acquire_actions(
        self,
        grade_name: str,
        property_id: str,
        rate_name: str,
        acquire_actions: Optional[List[AcquireAction]] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> MultiplyAcquireActionsByUserId:
        return MultiplyAcquireActionsByUserId(
            self.namespace_name,
            grade_name,
            property_id,
            rate_name,
            acquire_actions,
            user_id,
        )

    def sub_grade(
        self,
        grade_name: str,
        property_id: str,
        grade_value: Optional[int] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> SubGradeByUserId:
        return SubGradeByUserId(
            self.namespace_name,
            grade_name,
            property_id,
            grade_value,
            user_id,
        )

    def verify_grade(
        self,
        grade_name: str,
        verify_type: str,
        property_id: str,
        grade_value: Optional[int] = None,
        multiply_value_specifying_quantity: Optional[bool] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> VerifyGradeByUserId:
        return VerifyGradeByUserId(
            self.namespace_name,
            grade_name,
            verify_type,
            property_id,
            grade_value,
            multiply_value_specifying_quantity,
            user_id,
        )

    def verify_grade_up_material(
        self,
        grade_name: str,
        verify_type: str,
        property_id: str,
        material_property_id: str,
        user_id: Optional[str] = "#{userId}",
    ) -> VerifyGradeUpMaterialByUserId:
        return VerifyGradeUpMaterialByUserId(
            self.namespace_name,
            grade_name,
            verify_type,
            property_id,
            material_property_id,
            user_id,
        )

    def grn(
        self,
    ) -> str:
        return Join(
            ":",
            [
                "grn",
                "gs2",
                GetAttr.region(
                ).str(
                ),
                GetAttr.owner_id(
                ).str(
                ),
                "grade",
                self.namespace_name,
            ],
        ).str(
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .Version import Version
from .options.ScheduleVersionOptions import ScheduleVersionOptions


class ScheduleVersion:
    current_version: Version
    warning_version: Version
    error_version: Version
    schedule_event_id: Optional[str] = None

    def __init__(
        self,
        current_version: Version,
        warning_version: Version,
        error_version: Version,
        options: Optional[ScheduleVersionOptions] = ScheduleVersionOptions(),
    ):
        self.current_version = current_version
        self.warning_version = warning_version
        self.error_version = error_version
        self.schedule_event_id = options.schedule_event_id if options.schedule_event_id else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.current_version is not None:
            properties["currentVersion"] = self.current_version.properties(
            )
        if self.warning_version is not None:
            properties["warningVersion"] = self.warning_version.properties(
            )
        if self.error_version is not None:
            properties["errorVersion"] = self.error_version.properties(
            )
        if self.schedule_event_id is not None:
            properties["scheduleEventId"] = self.schedule_event_id

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
#
# deny overwrite
from __future__ import annotations
from typing import *

from ...core.func import GetAttr, Join
from .DailyTransactionHistoryRef import DailyTransactionHistoryRef
from .StoreContentModelRef import StoreContentModelRef
from .UnusedBalanceRef import UnusedBalanceRef
from ..stamp_sheet.DepositByUserId import DepositByUserId
from ..model.DepositTransaction import DepositTransaction
from ..stamp_sheet.WithdrawByUserId import WithdrawByUserId
from ..stamp_sheet.VerifyReceiptByUserId import VerifyReceiptByUserId
from ..model.Receipt import Receipt


class NamespaceRef:
    namespace_name: str

    def __init__(
        self,
        namespace_name: str,
    ):
        self.namespace_name = namespace_name

    def store_content_model(
        self,
        content_name: str,
    ) -> StoreContentModelRef:
        return StoreContentModelRef(
            self.namespace_name,
            content_name,
        )

    def deposit(
        self,
        slot: int,
        deposit_transactions: List[DepositTransaction],
        user_id: Optional[str] = "#{userId}",
    ) -> DepositByUserId:
        return DepositByUserId(
            self.namespace_name,
            slot,
            deposit_transactions,
            user_id,
        )

    def withdraw(
        self,
        slot: int,
        withdraw_count: int,
        paid_only: Optional[bool] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> WithdrawByUserId:
        return WithdrawByUserId(
            self.namespace_name,
            slot,
            withdraw_count,
            paid_only,
            user_id,
        )

    def verify_receipt(
        self,
        content_name: str,
        receipt: Optional[str] = "#{receipt}",
        user_id: Optional[str] = "#{userId}",
    ) -> VerifyReceiptByUserId:
        return VerifyReceiptByUserId(
            self.namespace_name,
            content_name,
            receipt,
            user_id,
        )

    def grn(
        self,
    ) -> str:
        return Join(
            ":",
            [
                "grn",
                "gs2",
                GetAttr.region(
                ).str(
                ),
                GetAttr.owner_id(
                ).str(
                ),
                "money2",
                self.namespace_name,
            ],
        ).str(
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.model import AcquireAction, ConsumeAction, VerifyAction
from ..model.BalanceParameterValue import BalanceParameterValue


class SetBalanceParameterStatusByUserId(AcquireAction):

    def __init__(
        self,
        namespace_name: str,
        parameter_name: str,
        property_id: str,
        parameter_values: List[BalanceParameterValue],
        user_id: Optional[str] = "#{userId}",
    ):
        properties: Dict[str, Any] = {}

        properties["namespaceName"] = namespace_name
        properties["parameterName"] = parameter_name
        properties["propertyId"] = property_id
        properties["parameterValues"] = parameter_values
        properties["userId"] = user_id

        super().__init__(
            "Gs2Enchant:SetBalanceParameterStatusByUserId",
            properties,
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .ItemModel import ItemModel
from .options.InventoryModelOptions import InventoryModelOptions


class InventoryModel:
    name: str
    initial_capacity: int
    max_capacity: int
    item_models: List[ItemModel]
    metadata: Optional[str] = None
    protect_referenced_item: Optional[bool] = None

    def __init__(
        self,
        name: str,
        initial_capacity: int,
        max_capacity: int,
        item_models: List[ItemModel],
        options: Optional[InventoryModelOptions] = InventoryModelOptions(),
    ):
        self.name = name
        self.initial_capacity = initial_capacity
        self.max_capacity = max_capacity
        self.item_models = item_models
        self.metadata = options.metadata if options.metadata else None
        self.protect_referenced_item = options.protect_referenced_item if options.protect_referenced_item else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.initial_capacity is not None:
            properties["initialCapacity"] = self.initial_capacity
        if self.max_capacity is not None:
            properties["maxCapacity"] = self.max_capacity
        if self.protect_referenced_item is not None:
            properties["protectReferencedItem"] = self.protect_referenced_item
        if self.item_models is not None:
            properties["itemModels"] = [
                v.properties(
                )
                for v in self.item_models
            ]

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from abc import abstractmethod
from typing import List, Dict

import gs2_cdk.script

def indent(value: str, size: int) -> str:
    return " " * size + value.replace("\n", "\n" + " " * size).strip(" ")


class Variable:

    def __init__(
            self,
            name: str,
    ):
        self.name = name

    @abstractmethod
    def type_name(self) -> str:
        pass

    def gsl(self):
        output = "{type} {name}".format(
            type=self.type_name(),
            name=self.name,
        )
        return output


class String(Variable):

    def __init__(
            self,
            name: str,
    ):
        super().__init__(
            name,
        )

    def type_name(self) -> str:
        return "string"


class Int(Variable):

    def __init__(
            self,
            name: str,
    ):
        super().__init__(
            name,
        )

    def type_name(self) -> str:
        return "int"


class Float(Variable):

    def __init__(
            self,
            name: str,
    ):
        super().__init__(
            name,
        )

    def type_name(self) -> str:
        return "float"


class Bool(Variable):

    def __init__(
            self,
            name: str,
    ):
        super().__init__(
            name,
        )

    def type_name(self) -> str:
        return "bool"


class Array(Variable):

    def __init__(
            self,
            name: str,
    ):
        super().__init__(
            name,
        )

    def type_name(self) -> str:
        return "array"


class Map(Variable):

    def __init__(
            self,
            name: str,
    ):
        super().__init__(
            name,
        )

    def type_name(self) -> str:
        return "map"


class Event:

    from_task_name: str

    def __init__(
            self,
            name: str,
            arguments: List[Variable],
            next_task_name: str,
    ):
        self.name = name
        self.arguments = arguments
        self.from_task_name = None
        self.next_task_name = next_task_name

    def gsl(self):
        output = ""
        output += "Transition {fromTaskName} handling {eventName} -> {nextTaskName};\n".format(
            fromTaskName=self.from_task_name,
            eventName=self.name,
            nextTaskName=self.next_task_name,
        )
        return output


class ErrorEvent(Event):

    def __init__(
            self,
            next_task_name: str,
    ):
        super().__init__(
            name="Error",
            arguments=[
                String(
                    name="reason",
                ),
            ],
            next_task_name=next_task_name
        )


class BaseTask:

    events: List[Event] = []

    def __init__(
            self,
            name: str,
            arguments: List[Variable],
    ):
        self.name = name
        self.arguments = arguments
        self.events = []

    @abstractmethod
    def gsl(self) -> str:
        pass

    @abstractmethod
    def mermaid(self) -> str:
        pass


class Result:

    def __init__(
            self,
            result_name: str,
            emit_event_name: str,
            emit_event_argument_variable_names: Dict[Variable, str],
    ):
        self.result_name = result_name
        self.emit_event_name = emit_event_name
        self.emit_event_argument_variable_names = emit_event_argument_variable_names


class Script:

    def __init__(
            self,
            name: str,
            payload: str,
    ):
        self.name = name
        self.payload = payload


class Task(BaseTask):

    results = []

    def __init__(
            self,
            name: str,
            arguments: List[Variable],
            script: str,
    ):
        super().__init__(
            name=name,
            arguments=arguments,
        )
        self.script = script
        self.results = []

    def transition(
            self,
            event: Event,
    ) -> 'Task':
        event.from_task_name = self.name
        self.events.append(event)
        return self

    def result(
            self,
            result_name: str,
            emit_event_argument_variable_names: Dict[Variable, str],
            next_task_name: str,
    ):
        self.results.append(
            Result(
                result_name=result_name,
                emit_event_name=result_name,
                emit_event_argument_variable_names=emit_event_argument_variable_names,
            )
        )
        self.transition(Event(
            name=result_name,
            arguments=[
                k
                for k, v in emit_event_argument_variable_names.items()
            ],
            next_task_name=next_task_name,
        ))
        return self

    def script_payload(self) -> Script:
        output = self.script
        output += "\n\n"
        for result in self.results:
            output += "if result == '{result_name}' then\n".format(
                result_name=result.result_name,
            )
            output += indent("""result = {{
    event='{emit_event_name}',
    params={{{param_names}}},
    updatedVariables=args.variables
}}
            """, 2).format(
                emit_event_name=result.emit_event_name,
                param_names=", ".join([
                    "{argument_name}={variable_name}".format(
                        argument_name=argument.name,
                        variable_name=variable_name,
                    )
                    for argument, variable_name in result.emit_event_argument_variable_names.items()
                ])
            )
            output += "end\n"
        return Script(
            name=self.name,
            payload=output,
        )

    def gsl(self):
        output = ""
        output += "Task {name}({arguments}) {{\n".format(
            name=self.name,
            arguments=", ".join(
                [
                    argument.gsl()
                    for argument in self.arguments
                ]
            ),
        )

        for event in self.events:
            output += indent("Event {event}({arguments});\n".format(
                event=event.name,
                arguments=", ".join(
                    [
                        argument.gsl()
                        for argument in event.arguments
                    ]
                ),
            ), 2)

        output += indent("Script grn:gs2:{{region}}:{{ownerId}}:script:{{scriptNamespaceName}}:script:{{stateMachineName}}_{taskName}\n".format(
            taskName=self.name,
        ), 2)

        output += "}\n\n"
        return output

    def mermaid(self):
        output = ""
        for event in self.events:
            if event.next_task_name == "Error":
                continue
            output += "{{stateMachineName}}_{from_task_name}[[{from_task_name}]] -->|{event_name}| {{stateMachineName}}_{next_task_name}\n".format(
                from_task_name=event.from_task_name,
                event_name=event.name,
                next_task_name=event.next_task_name,
            )
        return output


class InParam:
    def __init__(
            self,
            current_state_machine_variable: Variable,
            sub_state_machine_variable: Variable,
    ):
        self.current_state_machine_variable = current_state_machine_variable
        self.sub_state_machine_variable = sub_state_machine_variable


class OutParam:
    def __init__(
            self,
            sub_state_machine_variable: Variable,
            current_state_machine_variable: Variable,
    ):
        self.sub_state_machine_variable = sub_state_machine_variable
        self.current_state_machine_variable = current_state_machine_variable


class SubStateMachineTask(BaseTask):

    def __init__(
            self,
            name: str,
            sub_state_machine_name: str,
            in_params: List[InParam],
            out_params: List[OutParam],
    ):
        super().__init__(
            name=name,
            arguments=[],
        )
        self.sub_state_machine_name = sub_state_machine_name
        self.in_params = in_params
        self.out_params = out_params

    def transition(
            self,
            event: Event,
    ) -> 'SubStateMachineTask':
        event.from_task_name = self.name
        self.events.append(event)
        return self

    def gsl(self):
        output = ""
        output += "SubStateMachineTask {name} {{\n".format(
            name=self.name,
        )

        output += indent("using {subStateMachineName};\n".format(
            subStateMachineName=self.name,
        ), 2)

        output += indent("in (" + ", ".join([
            in_param.sub_state_machine_variable.name + " <- " + in_param.current_state_machine_variable.name
            for in_param in self.in_params
        ]) + ");\n", 2)

        output += indent("out (" + ", ".join([
            out_param.sub_state_machine_variable.name + " -> " + out_param.current_state_machine_variable.name
            for out_param in self.out_params
        ]) + ");\n", 2)

        output += "}\n\n"
        return output

    def mermaid(self):
        output = ""
        output += "{{stateMachineName}}_{name}[/{name}/]\n".format(
            name=self.name,
        )
        return output


class WaitTask(BaseTask):

    results = []

    def __init__(
            self,
            name: str,
    ):
        super().__init__(
            name=name,
            arguments=[],
        )

    def transition(
            self,
            event: Event,
    ) -> 'WaitTask':
        event.from_task_name = self.name
        self.events.append(event)
        return self

    def result(
            self,
            result_name: str,
            emit_event_argument_variable_names: Dict[Variable, str],
            next_task_name: str,
    ):
        self.results.append(
            Result(
                result_name=result_name,
                emit_event_name=result_name,
                emit_event_argument_variable_names=emit_event_argument_variable_names,
            )
        )
        self.transition(Event(
            name=result_name,
            arguments=[
                k
                for k, v in emit_event_argument_variable_names.items()
            ],
            next_task_name=next_task_name,
        ))
        return self

    def gsl(self):
        output = ""
        output += "WaitTask {name} {{\n".format(
            name=self.name,
        )

        for event in self.events:
            output += indent("Event {event}({arguments});\n".format(
                event=event.name,
                arguments=", ".join(
                    [
                        argument.gsl()
                        for argument in event.arguments
                    ]
                ),
                next=event.next_task_name,
            ), 2)

        output += "}\n\n"
        return output

    def mermaid(self):
        output = ""
        for event in self.events:
            if event.next_task_name == "Error":
                continue
            output += "{{stateMachineName}}_{from_task_name}([{from_task_name}]) -->|{event_name}| {{stateMachineName}}_{next_task_name}\n".format(
                from_task_name=event.from_task_name,
                event_name=event.name,
                next_task_name=event.next_task_name,
            )
        return output


class PassTask(BaseTask):

    def __init__(
            self,
            name: str,
    ):
        super().__init__(
            name=name,
            arguments=[],
        )

    def gsl(self):
        return "PassTask Pass;\n\n"

    def mermaid(self):
        output = ""
        output += "{{stateMachineName}}_{name}[\\{name}/]\n".format(
            name=self.name,
        )
        return output


class ErrorTask(BaseTask):

    def __init__(
            self,
            name: str,
    ):
        super().__init__(
            name=name,
            arguments=[
                String(
                    name="reason",
                ),
            ],
        )

    def gsl(self):
        return "ErrorTask {name}(string reason);\n\n".format(name=self.name)

    def mermaid(self):
        return ""


class StateMachine:

    def __init__(
            self,
            state_machine_definition: 'StateMachineDefinition',
            name: str,
            variables: List[Variable],
    ):
        self.name = name
        self.variables = variables
        self.tasks = []
        self.entry_point_value = None
        state_machine_definition.add(self)

    def task(
            self,
            *args,
    ) -> 'StateMachine':
        if isinstance(args, list) or isinstance(args, tuple):
            self.tasks.extend(list(args))
        else:
            self.tasks.append(args)
        return self

    def entry_point(
            self,
            task_name: str,
    ) -> 'StateMachine':
        self.entry_point_value = task_name
        return self

    def scripts(self):
        scripts = []
        for task in self.tasks:
            if isinstance(task, Task):
                script = task.script_payload()
                script.name = self.name + "_" + script.name
                scripts.append(script)
        return scripts

    def gsl(self):
        output = "StateMachine {name} {{\n".format(name=self.name)

        variables = ""
        if self.variables:
            variables += "Variables {\n"
        for variable in self.variables:
            variables += indent(variable.gsl() + ";\n", 2)
        if self.variables:
            variables += "}\n\n"

        output += indent(variables, 2)

        if self.entry_point_value:
            output += "  EntryPoint {entry_point};\n\n".format(
                entry_point=self.entry_point_value,
            )

        for task in self.tasks:
            output += indent(task.gsl(), 2)

        for task in self.tasks:
            for event in task.events:
                output += indent(event.gsl(), 2)

        output += "}\n"

        output = output.replace("{stateMachineName}", self.name)

        return output

    def mermaid(self):
        output = "subgraph {name}\n".format(name=self.name)
        for task in self.tasks:
            output += indent(task.mermaid(), 2)
            output += "\n"
        output += "end\n"

        for task in self.tasks:
            if isinstance(task, SubStateMachineTask):
                output += "\n"
                output += "{{stateMachineName}}_{taskName} --> {subStateMachineName}_{{{subStateMachineName}_entryPoint}}\n".format(
                    taskName=task.name,
                    subStateMachineName=task.sub_state_machine_name,
                )
                output += "{subStateMachineName}_Pass -->|Pass| {{stateMachineName}}_{nextTaskName}\n".format(
                    subStateMachineName=task.sub_state_machine_name,
                    nextTaskName=task.events[0].next_task_name,
                )
            if isinstance(task, WaitTask):
                output += "\n"
                output += "Player ----->|Interaction| {{stateMachineName}}_{taskName}\n".format(
                    taskName=task.name,
                )

        output = output.replace("{stateMachineName}", self.name)

        return output


class StateMachineDefinition:

    state_machine_name: str = ""
    state_machines: List[StateMachine] = []

    def add(
            self,
            state_machine: StateMachine,
    ):
        self.state_machines.append(state_machine)

    def entry_point_state_machine_name(
            self,
            state_machine_name: str,
    ):
        self.state_machine_name = state_machine_name

    def state_machine(
            self,
            name: str,
            variables: List[Variable],
    ) -> StateMachine:
        return StateMachine(
            state_machine_definition=self,
            name=name,
            variables=variables,
        )

    def script_task(
            self,
            name: str,
            arguments: List[Variable],
            script: str,
    ) -> Task:
        return Task(
            name=name,
            arguments=arguments,
            script=script,
        )

    def sub_state_machine_task(
            self,
            name: str,
            sub_state_machine_name: str,
            in_params: List[InParam],
            out_params: List[OutParam],
            next_task_name: str,
    ) -> SubStateMachineTask:
        return SubStateMachineTask(
            name=sub_state_machine_name,
            sub_state_machine_name=sub_state_machine_name,
            in_params=in_params,
            out_params=out_params,
        ).transition(
            Event(
                name="Pass",
                arguments=[
                    out_param.sub_state_machine_variable
                    for out_param in out_params
                ],
                next_task_name=next_task_name,
            )
        )

    def in_param(
            self,
            current_state_machine_variable: Variable,
            sub_state_machine_variable: Variable,
    ) -> InParam:
        return InParam(
            current_state_machine_variable,
            sub_state_machine_variable,
        )

    def out_param(
            self,
            sub_state_machine_variable: Variable,
            current_state_machine_variable: Variable,
    ) -> OutParam:
        return OutParam(
            sub_state_machine_variable,
            current_state_machine_variable,
        )

    def wait_task(
            self,
            name: str,
    ) -> WaitTask:
        return WaitTask(
            name=name,
        )

    def pass_task(
            self,
            name: str,
    ) -> PassTask:
        return PassTask(
            name=name,
        )

    def error_task(
            self,
            name: str,
    ) -> ErrorTask:
        return ErrorTask(
            name=name,
        )

    def result(
            self,
            name: str,
            event_name: str,
            event_argument_variable_names: Dict[Variable, str],
    ) -> Result:
        return Result(
            result_name=name,
            emit_event_name=event_name,
            emit_event_argument_variable_names=event_argument_variable_names,
        )

    def string_type(
            self,
            name: str,
    ) -> String:
        return String(
            name=name,
        )

    def int_type(
            self,
            name: str,
    ) -> Int:
        return Int(
            name=name,
        )

    def float_type(
            self,
            name: str,
    ) -> Float:
        return Float(
            name=name,
        )

    def bool_type(
            self,
            name: str,
    ) -> Bool:
        return Bool(
            name=name,
        )

    def array_type(
            self,
            name: str,
    ) -> Array:
        return Array(
            name=name,
        )

    def map_type(
            self,
            name: str,
    ) -> Map:
        return Map(
            name=name,
        )

    def append_scripts(
            self,
            stack: "gs2_cdk.Stack",
            script_namespace: gs2_cdk.script.Namespace,
    ) -> List["gs2_cdk.script.Script"]:
        scripts = []
        for state_machines in self.state_machines:
            for script in state_machines.scripts():
                deploy_script = gs2_cdk.script.Script(
                    stack=stack,
                    namespace_name=script_namespace.name,
                    name=script.name,
                    script=script.payload.strip(),
                )
                deploy_script.add_depends_on(script_namespace)
                scripts.append(deploy_script)
        return scripts

    def gsl(self):
        output = ""
        for state_machine in self.state_machines:
            output += state_machine.gsl()
            output += "\n"
        return output

    def mermaid(self):
        output = "flowchart TD\n"

        output += indent("Start ----> {stateMachineName}_{entryPoint}\n".format(
            stateMachineName=self.state_machines[0].name,
            entryPoint=self.state_machines[0].entry_point_value,
        ), 2)

        output += indent("{stateMachineName}_Pass ----> Exit\n".format(
            stateMachineName=self.state_machines[0].name,
        ), 2)

        for state_machine in self.state_machines:
            output += indent(state_machine.mermaid(), 2)
            output += "\n"

        for state_machine in self.state_machines:
            output = output.replace("{" + state_machine.name + "_entryPoint}", state_machine.entry_point_value)
        return output



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .AppleAppStoreVerifyReceiptEvent import AppleAppStoreVerifyReceiptEvent
from .GooglePlayVerifyReceiptEvent import GooglePlayVerifyReceiptEvent
from .options.VerifyReceiptEventOptions import VerifyReceiptEventOptions
from .enum.VerifyReceiptEventPlatform import VerifyReceiptEventPlatform


class VerifyReceiptEvent:
    content_name: str
    platform: VerifyReceiptEventPlatform
    apple_app_store_verify_receipt_event: Optional[AppleAppStoreVerifyReceiptEvent] = None
    google_play_verify_receipt_event: Optional[GooglePlayVerifyReceiptEvent] = None

    def __init__(
        self,
        content_name: str,
        platform: VerifyReceiptEventPlatform,
        options: Optional[VerifyReceiptEventOptions] = VerifyReceiptEventOptions(),
    ):
        self.content_name = content_name
        self.platform = platform
        self.apple_app_store_verify_receipt_event = options.apple_app_store_verify_receipt_event if options.apple_app_store_verify_receipt_event else None
        self.google_play_verify_receipt_event = options.google_play_verify_receipt_event if options.google_play_verify_receipt_event else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.content_name is not None:
            properties["contentName"] = self.content_name
        if self.platform is not None:
            properties["platform"] = self.platform.value
        if self.apple_app_store_verify_receipt_event is not None:
            properties["appleAppStoreVerifyReceiptEvent"] = self.apple_app_store_verify_receipt_event.properties(
            )
        if self.google_play_verify_receipt_event is not None:
            properties["googlePlayVerifyReceiptEvent"] = self.google_play_verify_receipt_event.properties(
            )

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .NamespaceRef import NamespaceRef
from .GuildModelRef import GuildModelRef
from .InboxRef import InboxRef
from .GuildRef import GuildRef
from .IgnoreUsersRef import IgnoreUsersRef
from .LastGuildMasterActivityRef import LastGuildMasterActivityRef
from .RoleModelRef import RoleModelRef
from .IgnoreUserRef import IgnoreUserRef


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .SalesItem import SalesItem
from .options.SalesItemOptions import SalesItemOptions
from .SalesItemGroup import SalesItemGroup
from .options.SalesItemGroupOptions import SalesItemGroupOptions
from .Showcase import Showcase
from .options.ShowcaseOptions import ShowcaseOptions
from .DisplayItem import DisplayItem
from .options.DisplayItemOptions import DisplayItemOptions
from .enum.DisplayItemType import DisplayItemType
from .options.DisplayItemTypeIsSalesItemOptions import DisplayItemTypeIsSalesItemOptions
from .options.DisplayItemTypeIsSalesItemGroupOptions import DisplayItemTypeIsSalesItemGroupOptions
from .RandomShowcase import RandomShowcase
from .options.RandomShowcaseOptions import RandomShowcaseOptions
from .PurchaseCount import PurchaseCount
from .options.PurchaseCountOptions import PurchaseCountOptions
from .RandomDisplayItemModel import RandomDisplayItemModel
from .options.RandomDisplayItemModelOptions import RandomDisplayItemModelOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from ...core.model import AcquireAction
from .AcquireActionList import AcquireActionList
from .options.CategoryModelOptions import CategoryModelOptions


class CategoryModel:
    name: str
    reward_interval_minutes: int
    default_maximum_idle_minutes: int
    acquire_actions: List[AcquireActionList]
    metadata: Optional[str] = None
    idle_period_schedule_id: Optional[str] = None
    receive_period_schedule_id: Optional[str] = None

    def __init__(
        self,
        name: str,
        reward_interval_minutes: int,
        default_maximum_idle_minutes: int,
        acquire_actions: List[AcquireActionList],
        options: Optional[CategoryModelOptions] = CategoryModelOptions(),
    ):
        self.name = name
        self.reward_interval_minutes = reward_interval_minutes
        self.default_maximum_idle_minutes = default_maximum_idle_minutes
        self.acquire_actions = acquire_actions
        self.metadata = options.metadata if options.metadata else None
        self.idle_period_schedule_id = options.idle_period_schedule_id if options.idle_period_schedule_id else None
        self.receive_period_schedule_id = options.receive_period_schedule_id if options.receive_period_schedule_id else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.reward_interval_minutes is not None:
            properties["rewardIntervalMinutes"] = self.reward_interval_minutes
        if self.default_maximum_idle_minutes is not None:
            properties["defaultMaximumIdleMinutes"] = self.default_maximum_idle_minutes
        if self.acquire_actions is not None:
            properties["acquireActions"] = [
                v.properties(
                )
                for v in self.acquire_actions
            ]
        if self.idle_period_schedule_id is not None:
            properties["idlePeriodScheduleId"] = self.idle_period_schedule_id
        if self.receive_period_schedule_id is not None:
            properties["receivePeriodScheduleId"] = self.receive_period_schedule_id

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from ...core.model import ConsumeAction
from ...core.model import AcquireAction
from .options.StampSheetResultOptions import StampSheetResultOptions


class StampSheetResult:
    user_id: str
    transaction_id: str
    sheet_request: AcquireAction
    task_requests: Optional[List[ConsumeAction]] = None
    task_result_codes: Optional[List[int]] = None
    task_results: Optional[List[str]] = None
    sheet_result_code: Optional[int] = None
    sheet_result: Optional[str] = None
    next_transaction_id: Optional[str] = None
    revision: Optional[int] = None

    def __init__(
        self,
        user_id: str,
        transaction_id: str,
        sheet_request: AcquireAction,
        options: Optional[StampSheetResultOptions] = StampSheetResultOptions(),
    ):
        self.user_id = user_id
        self.transaction_id = transaction_id
        self.sheet_request = sheet_request
        self.task_requests = options.task_requests if options.task_requests else None
        self.task_result_codes = options.task_result_codes if options.task_result_codes else None
        self.task_results = options.task_results if options.task_results else None
        self.sheet_result_code = options.sheet_result_code if options.sheet_result_code else None
        self.sheet_result = options.sheet_result if options.sheet_result else None
        self.next_transaction_id = options.next_transaction_id if options.next_transaction_id else None
        self.revision = options.revision if options.revision else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.user_id is not None:
            properties["userId"] = self.user_id
        if self.transaction_id is not None:
            properties["transactionId"] = self.transaction_id
        if self.task_requests is not None:
            properties["taskRequests"] = [
                v.properties(
                )
                for v in self.task_requests
            ]
        if self.sheet_request is not None:
            properties["sheetRequest"] = self.sheet_request.properties(
            )
        if self.task_result_codes is not None:
            properties["taskResultCodes"] = self.task_result_codes
        if self.task_results is not None:
            properties["taskResults"] = self.task_results
        if self.sheet_result_code is not None:
            properties["sheetResultCode"] = self.sheet_result_code
        if self.sheet_result is not None:
            properties["sheetResult"] = self.sheet_result
        if self.next_transaction_id is not None:
            properties["nextTransactionId"] = self.next_transaction_id

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .RepeatSetting import RepeatSetting
from .options.EventOptions import EventOptions
from .options.EventScheduleTypeIsAbsoluteOptions import EventScheduleTypeIsAbsoluteOptions
from .options.EventScheduleTypeIsRelativeOptions import EventScheduleTypeIsRelativeOptions
from .options.EventRepeatTypeIsAlwaysOptions import EventRepeatTypeIsAlwaysOptions
from .options.EventRepeatTypeIsDailyOptions import EventRepeatTypeIsDailyOptions
from .options.EventRepeatTypeIsWeeklyOptions import EventRepeatTypeIsWeeklyOptions
from .options.EventRepeatTypeIsMonthlyOptions import EventRepeatTypeIsMonthlyOptions
from .enum.EventScheduleType import EventScheduleType
from .enum.EventRepeatType import EventRepeatType
from .enum.EventRepeatBeginDayOfWeek import EventRepeatBeginDayOfWeek
from .enum.EventRepeatEndDayOfWeek import EventRepeatEndDayOfWeek


class Event:
    name: str
    schedule_type: EventScheduleType
    repeat_setting: RepeatSetting
    repeat_type: EventRepeatType
    metadata: Optional[str] = None
    absolute_begin: Optional[int] = None
    absolute_end: Optional[int] = None
    relative_trigger_name: Optional[str] = None
    repeat_begin_day_of_month: Optional[int] = None
    repeat_end_day_of_month: Optional[int] = None
    repeat_begin_day_of_week: Optional[EventRepeatBeginDayOfWeek] = None
    repeat_end_day_of_week: Optional[EventRepeatEndDayOfWeek] = None
    repeat_begin_hour: Optional[int] = None
    repeat_end_hour: Optional[int] = None

    def __init__(
        self,
        name: str,
        schedule_type: EventScheduleType,
        repeat_setting: RepeatSetting,
        repeat_type: EventRepeatType,
        options: Optional[EventOptions] = EventOptions(),
    ):
        self.name = name
        self.schedule_type = schedule_type
        self.repeat_setting = repeat_setting
        self.repeat_type = repeat_type
        self.metadata = options.metadata if options.metadata else None
        self.absolute_begin = options.absolute_begin if options.absolute_begin else None
        self.absolute_end = options.absolute_end if options.absolute_end else None
        self.relative_trigger_name = options.relative_trigger_name if options.relative_trigger_name else None
        self.repeat_begin_day_of_month = options.repeat_begin_day_of_month if options.repeat_begin_day_of_month else None
        self.repeat_end_day_of_month = options.repeat_end_day_of_month if options.repeat_end_day_of_month else None
        self.repeat_begin_day_of_week = options.repeat_begin_day_of_week if options.repeat_begin_day_of_week else None
        self.repeat_end_day_of_week = options.repeat_end_day_of_week if options.repeat_end_day_of_week else None
        self.repeat_begin_hour = options.repeat_begin_hour if options.repeat_begin_hour else None
        self.repeat_end_hour = options.repeat_end_hour if options.repeat_end_hour else None

    @staticmethod
    def schedule_type_is_absolute(
        name: str,
        repeat_setting: RepeatSetting,
        repeat_type: EventRepeatType,
        options: Optional[EventScheduleTypeIsAbsoluteOptions] = EventScheduleTypeIsAbsoluteOptions(),
    ) -> Event:
        return Event(
            name,
            EventScheduleType.ABSOLUTE,
            repeat_setting,
            repeat_type,
            EventOptions(
                options.metadata,
                options.absolute_begin,
                options.absolute_end,
            ),
        )

    @staticmethod
    def schedule_type_is_relative(
        name: str,
        repeat_setting: RepeatSetting,
        repeat_type: EventRepeatType,
        relative_trigger_name: str,
        options: Optional[EventScheduleTypeIsRelativeOptions] = EventScheduleTypeIsRelativeOptions(),
    ) -> Event:
        return Event(
            name,
            EventScheduleType.RELATIVE,
            repeat_setting,
            repeat_type,
            EventOptions(
                relative_trigger_name,
                options.metadata,
                options.absolute_begin,
                options.absolute_end,
            ),
        )

    @staticmethod
    def repeat_type_is_always(
        name: str,
        schedule_type: EventScheduleType,
        repeat_setting: RepeatSetting,
        options: Optional[EventRepeatTypeIsAlwaysOptions] = EventRepeatTypeIsAlwaysOptions(),
    ) -> Event:
        return Event(
            name,
            schedule_type,
            repeat_setting,
            EventRepeatType.ALWAYS,
            EventOptions(
                options.metadata,
                options.absolute_begin,
                options.absolute_end,
            ),
        )

    @staticmethod
    def repeat_type_is_daily(
        name: str,
        schedule_type: EventScheduleType,
        repeat_setting: RepeatSetting,
        repeat_begin_hour: int,
        repeat_end_hour: int,
        options: Optional[EventRepeatTypeIsDailyOptions] = EventRepeatTypeIsDailyOptions(),
    ) -> Event:
        return Event(
            name,
            schedule_type,
            repeat_setting,
            EventRepeatType.DAILY,
            EventOptions(
                repeat_begin_hour,
                repeat_end_hour,
                options.metadata,
                options.absolute_begin,
                options.absolute_end,
            ),
        )

    @staticmethod
    def repeat_type_is_weekly(
        name: str,
        schedule_type: EventScheduleType,
        repeat_setting: RepeatSetting,
        repeat_begin_day_of_week: EventRepeatBeginDayOfWeek,
        repeat_end_day_of_week: EventRepeatEndDayOfWeek,
        repeat_begin_hour: int,
        repeat_end_hour: int,
        options: Optional[EventRepeatTypeIsWeeklyOptions] = EventRepeatTypeIsWeeklyOptions(),
    ) -> Event:
        return Event(
            name,
            schedule_type,
            repeat_setting,
            EventRepeatType.WEEKLY,
            EventOptions(
                repeat_begin_day_of_week,
                repeat_end_day_of_week,
                repeat_begin_hour,
                repeat_end_hour,
                options.metadata,
                options.absolute_begin,
                options.absolute_end,
            ),
        )

    @staticmethod
    def repeat_type_is_monthly(
        name: str,
        schedule_type: EventScheduleType,
        repeat_setting: RepeatSetting,
        repeat_begin_day_of_month: int,
        repeat_end_day_of_month: int,
        repeat_begin_hour: int,
        repeat_end_hour: int,
        options: Optional[EventRepeatTypeIsMonthlyOptions] = EventRepeatTypeIsMonthlyOptions(),
    ) -> Event:
        return Event(
            name,
            schedule_type,
            repeat_setting,
            EventRepeatType.MONTHLY,
            EventOptions(
                repeat_begin_day_of_month,
                repeat_end_day_of_month,
                repeat_begin_hour,
                repeat_end_hour,
                options.metadata,
                options.absolute_begin,
                options.absolute_end,
            ),
        )

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.schedule_type is not None:
            properties["scheduleType"] = self.schedule_type.value
        if self.absolute_begin is not None:
            properties["absoluteBegin"] = self.absolute_begin
        if self.absolute_end is not None:
            properties["absoluteEnd"] = self.absolute_end
        if self.relative_trigger_name is not None:
            properties["relativeTriggerName"] = self.relative_trigger_name
        if self.repeat_setting is not None:
            properties["repeatSetting"] = self.repeat_setting.properties(
            )
        if self.repeat_type is not None:
            properties["repeatType"] = self.repeat_type.value
        if self.repeat_begin_day_of_month is not None:
            properties["repeatBeginDayOfMonth"] = self.repeat_begin_day_of_month
        if self.repeat_end_day_of_month is not None:
            properties["repeatEndDayOfMonth"] = self.repeat_end_day_of_month
        if self.repeat_begin_day_of_week is not None:
            properties["repeatBeginDayOfWeek"] = self.repeat_begin_day_of_week.value
        if self.repeat_end_day_of_week is not None:
            properties["repeatEndDayOfWeek"] = self.repeat_end_day_of_week.value
        if self.repeat_begin_hour is not None:
            properties["repeatBeginHour"] = self.repeat_begin_hour
        if self.repeat_end_hour is not None:
            properties["repeatEndHour"] = self.repeat_end_hour

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from .Statement import Statement
from ...core.func import GetAttr, Join


class Policy:

    version: str = "2016-04-01"
    statements: List[Statement]

    def __init__(
            self,
            statements: List[Statement]
    ):
        self.statements = statements

    def properties(self) -> Dict[str, Any]:
        return {
            "Version": self.version,
            "Statements": [
                statement.properties()
                for statement in self.statements
            ],
        }
